"""Techs module - Technology editing functionality."""

from aoe2_genie_tooling.Techs.tech_manager import TechManager
from aoe2_genie_tooling.Techs.tech_handle import TechHandle

__all__ = ["TechManager", "TechHandle"]
