"""Sounds module - Sound editing functionality."""

from aoe2_genie_tooling.Sounds.sound_manager import SoundManager
from aoe2_genie_tooling.Sounds.sound_handle import SoundHandle

__all__ = ["SoundManager", "SoundHandle"]
