from enum import IntEnum


class UnitType(IntEnum):
    """
    Genie Editor Unit Types.
    
    Source: https://ageofempires.fandom.com/wiki/Genie_Editor#Unit_type
    """
    EYE_CANDY = 10
    TREES_AOK = 15
    ANIMATED = 20
    DOPPELGANGER = 25
    MOVING = 30
    ACTOR = 40
    SUPERCLASS = 50
    PROJECTILE = 60
    COMBATANT = 70
    BUILDING = 80
    TREE_AOE = 90
