from __future__ import annotations

from enum import IntEnum


class UnitType(IntEnum):
    EyeCandy = 10
    Trees = 15
    Animated = 20
    Doppelganger = 25
    Moving = 30
    Acting = 40
    Combat = 50
    Projectile = 60
    Creatable = 70
    Building = 80
    LegacyTree = 90
